package javax.print;

import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.util.Iterator;
import java.util.ServiceConfigurationError;
import java.util.ServiceLoader;
import java.security.PrivilegedExceptionAction;
import java.io.OutputStream;
import java.util.ArrayList;
import sun.awt.AppContext;

public abstract class StreamPrintServiceFactory
{
    public StreamPrintServiceFactory() {
    }
    
    private static StreamPrintServiceFactory.Services getServices() {
        StreamPrintServiceFactory.Services value = (StreamPrintServiceFactory.Services)AppContext.getAppContext().get(StreamPrintServiceFactory.Services.class);
        if (value == null) {
            value = new StreamPrintServiceFactory.Services();
            AppContext.getAppContext().put(StreamPrintServiceFactory.Services.class, value);
            return value;
        }
        return value;
    }
    
    private static ArrayList getListOfFactories() {
        return getServices().listOfFactories;
    }
    
    private static ArrayList initListOfFactories() {
        ArrayList<Object> list = new ArrayList();
        getServices().listOfFactories = list;
        return list;
    }
    
    public static StreamPrintServiceFactory[] lookupStreamPrintServiceFactories(DocFlavor docFlavor, String s) {
        ArrayList<Object> factories = getFactories(docFlavor, s);
        return (StreamPrintServiceFactory[])(StreamPrintServiceFactory[])factories.toArray(new StreamPrintServiceFactory[factories.size()]);
    }
    
    public abstract String getOutputFormat();
    
    public abstract DocFlavor[] getSupportedDocFlavors();
    
    public abstract StreamPrintService getPrintService(OutputStream p0);
    
    private static ArrayList getAllFactories() {
        synchronized (StreamPrintServiceFactory.class) {
            ArrayList<Object> listOfFactories = getListOfFactories();
            if (listOfFactories != null) {
                return listOfFactories;
            }
            ArrayList<Object> initListOfFactories = initListOfFactories();
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction() {
                    @Override
                    public Object run() {
                        Iterator<StreamPrintServiceFactory> iterator = ServiceLoader.load(StreamPrintServiceFactory.class).iterator();
                        ArrayList<Object> access$100 = StreamPrintServiceFactory.access$100();
                        while (iterator.hasNext()) {
                            try {
                                access$100.add(iterator.next());
                            }
                            catch (ServiceConfigurationError serviceConfigurationError) {
                                if (System.getSecurityManager() != null) {
                                    serviceConfigurationError.printStackTrace();
                                }
                                else {
                                    throw serviceConfigurationError;
                                }
                            }
                        }
                        return null;
                    }
                });
            }
            catch (PrivilegedActionException ex) {}
            return initListOfFactories;
        }
    }
    
    private static boolean isMember(DocFlavor docFlavor, DocFlavor[] array) {
        int i = 0;
        while (i < array.length) {
            if (docFlavor.equals(array[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static ArrayList getFactories(DocFlavor docFlavor, String s) {
        if (docFlavor == null && s == null) {
            return getAllFactories();
        }
        ArrayList<StreamPrintServiceFactory> list = new ArrayList();
        for (StreamPrintServiceFactory e : getAllFactories()) {
            if ((s == null || s.equalsIgnoreCase(e.getOutputFormat())) && (docFlavor == null || isMember(docFlavor, e.getSupportedDocFlavors()))) {
                list.add(e);
            }
            else {
                continue;
            }
        }
        return list;
    }
    
    static ArrayList access$100() {
        return getListOfFactories();
    }
    
    static class Services
    {
        private ArrayList listOfFactories;
        
        Services() {
            this.listOfFactories = null;
        }
    }
}
